/*File: core8.js
 * Defines the objects used in the tools and all the functions that are called 
 * by the tools. By making all the functions etc methods of this it removes any 
 * possibility of conflict with fenctions defined by the in-game scripts.
 * 
 * The files below are loaded to give specific functionality.
 * 
 * idb3.js			- Code to manage the IndexedDB used by the tools.
 * inventory.js		- Code for managing the town and hub inventories.
 * military.js		- Code to collect, store and display information about troops
 *						and commanders.
 * maptools.js		- Scan and collect information from the world map.
 * mail.js			- Allow an in-game mail to be sent to more than one recipient.
 * crafting.js		- Information on crafted items.
 * quests.js			- Tools to help with in-game quests.
 * data.js			- Holds various sets of data that are used in the tools.
 * diplo.js			- This is where the functions for showing the number of diplo 
 *						units will live.
 * trade.js			- This is where the functions for showing the number of trade 
 *						units will live.
 * data.js			- Data objects used by a variety of the functions.
 * quests.js			- Functions relating to the in-game quests.
 * crafting.js		- Functions relating to crafting items.
 * mail.js			- Functions for allowing a message to be sent to more than one
 *						recipient.
 * buildings.js		- Functions for scanning the town map, displaying a towns 
 *						buildings and searching all towns for a building.
 * sovereignty.js	- Holds information on sovereignty bonuses in-game and methods 
 *						to find and store the ones available to each town.
 * mailsearch.js	- Functions for searching the users inbox. Data is stored in a
 *						IndexedDB.
 
 */
/* Class: dt
 * Container for all data and functions relating to the tools.
 * 
 * Properties:
 * server - string. Url for the server. This is the server that all scripts and 
 * resources will be served from.
 * factionScanState - boolean. Indicates if faction troop sightings should be 
 * reported or not.
 * processingHash - boolean. Indicates if the script is in the middle of processing a hash change.
 * db - object. IndexedDB object.
 */
dt = {
	server: 'https://www.durctools3dev.dewt.me.uk',
	factionScanState: false,
	processingHash: false,
	db: null,
	mdb: null,
	oldH: '',
	/*Object: player
	 * Holds all data relating to a player
	 * 
	 * Properties:
	 * name - string. The name of the player.
	 * alliance - integer. The id of the alliance this player belongs to. Is 0 if not in an alliance.
	 * towns - object. Data relating to the towns owned by this player. See <town> for details of the structure.
	 * hubs - object. Data relating to the hubs the player has goods stored at. See <hubs> for details of the structure.
	 * settings - object. Settings for various tools. See <settings> for details of the structure.
	 */
	player: function (name, alliance, towns, hubs, settings, pres) {
		this.name = name;
		this.alliance = alliance;
		this.towns = towns;
		this.hubs = hubs;
		this.settings = settings;
	},
	/* Object: town
	 * Holds all data relating to a particular town.
	 * 
	 * Properties:
	 * name - string. The name of the town.
	 * x - integer. X coordinate of the town.
	 * y - integer. Y coordinate of the town.
	 * inv - object. Details of the inventory held by this town. See <inv> for details of the structure.
	 * alembine - object. Status of this towns visits to each of the Allembine sites.
	 * mil - object. Details of the troops currently in this town or being built there and the attack/defence scores for the town. See <mil> for details of the structure.
	 * diplo - object. Details of the diplomatic units currently in this town as well as the diplomatic defence scores for this town. See <diplo> for details of the structure.
	 * trade - details of the trade units currently in this town. See <trade> for details of the structure.
	 */
	town: function (name, x, y, inv, allembine, mil, diplo, trade, prod) {
		this.name = name;
		this.x = x;
		this.y = y;
		this.inv = inv;
		this.allembine = allembine;
		this.mil = mil;
		this.diplo = diplo;
		this.trade = trade;
		this.prod = prod;
	},
	/* Object: hubs
	 * Holds all data relating to a particular trade hub the user has goods and/or trade units at.
	 * 
	 * Properties:
	 * name - The name of the hub.
	 * x - integer. The x coordinate of the hub.
	 * y - integer. The y coordinate of the hub.
	 * traders - unknown. Details of the traders at this hub.
	 * vans - unknown. Details of the caravans at this hub.
	 * inv - object. Details of the items currently held at this hub.
	 */
	hubs: function (name, x, y, traders, vans, inv) {
		this.name = name;
		this.x = x;
		this.y = y;
		this.inv = inv;
		this.traders = traders;
		this.vans = vans;
	},
	/* Object: inv
	 * Holds all data on the items held in the town. The items are organized into the same categories as the tabs at the bottom of the Inventory page.
	 * 
	 * Properties:
	 * maxStore - integer. The maximum storage capacity in this town.
	 * res - object. Details of base and advanced resources held in this town. For each resource a property is created which is an instance of the <item> object.
	 * mounts - object. Details of any specialist mounts held in this town. For each mount type a property is created which is an instance of the <item> object.
	 * weapons - object. Details of any specialist weapons held in this town. For each weapon a property is created which is an instance of the <item> object.
	 * armour - object. Details of any specialist armour held in this town. For each armour a property is created which is an instance of the <item> object.
	 * anatomies - object. Details of any animal parts and hides held in this town. For each anatomy a property is created which is an instance of the <item> object.
	 * minerals - object. Details of any minerals (rare and common) held in this town. For each mineral a property is created which is an instance of the <item> object.
	 * herbs - object. Details of any herbs (rare and common) held in this town. For each herb a property is created which is an instance of the <item> object.
	 * exotic - object. Details of any exotic items held in this town. For each exotic item a property is created which is an instance of the <item> object.
	 */
	inv: function (maxStore, res, mounts, weapons, armour, anatomies, minerals, herbs, exotic) {
		this.maxStore = maxStore;
		this.res = res;
		this.mounts = mounts;
		this.weapons = weapons;
		this.armour = armour;
		this.anatomies = anatomies;
		this.minerals = minerals;
		this.herbs = herbs;
		this.exotic = exotic;
	},
	/* Object: item
	 * Holds all the data for a particular item.
	 * 
	 * Parameters:
	 * name - string. The name of the item.
	 * held - integer. The quantity of this item held.
	 * classes - string. The classes used to display the icon/image for this item.
	 * id - string. String that is used to identify this item in various parts of the DOM.
	 * rate - float. The amount the quantity of this item changes each hour. Is 0 for items that do not have an hourly rate.
	 * 
	 */
	item: function (name, held, classes, id, rate) {
		this.name = name;
		this.held = held;
		this.classes = classes;
		this.id = id;
		this.rate = rate;
	},
	/* Object: trade
	 * Holds the number of each trade unit in a town.
	 * 
	 * Properties:
	 * van - integer. The number of caravans currently in this town.
	 * cotter - integer. The number of cotters currently in this town.
	 * herbalist - integer. The number of herbalists currently in this town.
	 * miner - integer. The number of miners currently in this town.
	 * skinner - integer. The number of skinners currently in this town.
	 * 
	 */
	trade: function (van, cotter, herbalist, miner, skinner, trader) {
		this.van = van;
		this.cotter = cotter;
		this.herbalist = herbalist;
		this.miner = miner;
		this.skinner = skinner;
		this.trader = trader;
	},
	/* Object: diplo
	 * Holds the number of each diplo unit in a town;
	 * 
	 * Properties:
	 * scout - array. The number of T1 and T2 scouts that are in this town.
	 * spy - array. The number of T1 and T2 spies that are in this town.
	 * thief - array. The number of T1 and T2 thieves that are in this town.
	 * saboteur - array. The number of T1 and T2 saboteurs that are in this town.
	 * assassin - array. The number of T1 and T2 assassins that are in this town.
	 * messenger - array. The number of messengers that are in this town.
	 */
	diplo: function (scout, spy, thief, saboteur, assassin, messenger) {
		this.scout = scout;
		this.spy = spy;
		this.thief = thief;
		this.saboteur = saboteur;
		this.assassin = assassin;
		this.messenger = messenger;
	},
	/* *************************************************************************
	 * Methods
	 ************************************************************************* */

	/* Method: addElement
	 * Create and append an element with multiple attributes
	 *
	 * Parameters:
	 * parent - the element the element is to be appended to
	 * elementType - the type of element to add.
	 * attribVal - array holding the values to be given to the attributes
	 * attribName - array holding the names of the attributes to be added.
	 *
	 * Returns:
	 * The new element.
	 */
	addElement: function (assignTo, parent, elementType, attribNames, attribVals) {
		var newElem = document.createElement(elementType);
		if (attribNames) {
			var numAttribs = attribNames.length;
			if (numAttribs > 0) {
				for (var i = 0; i < numAttribs; i++) {
					switch(attribNames[i]){
						case 'textContent':
							newElem.textContent = attribVals[i];
							break;
						case 'innerHTML':
							newElem.innerHTML = attribVals[i];
							break;
						default:
					newElem.setAttribute(attribNames[i], attribVals[i]);
							break;
					}
//					newElem.setAttribute(attribNames[i], attribVals[i]);
				}
			}
		}
		if(document.querySelector(parent) !== null){
		var newDiv = document.querySelector(parent).appendChild(newElem);
	}else{
		window.alert(parent);
	}
//		var newDiv = parent.appendChild(newElem);
		if (assignTo !== '') {
			dt[assignTo] = newDiv;
		}
		return newDiv;
	},
	/* Method: addElement2
 * Create and append an element with multiple attributes
 *
 * Parameters:
 * parent - the element the element is to be appended to
 * elementType - the type of element to add.
 * attribVal - array holding the values to be given to the attributes
 * attribName - array holding the names of the attributes to be added.
 *
 * Returns:
 * The new element.
 */
addElement2: function(parent, elementType, attribName, attribVal) {
	var newElem = document.createElement(elementType);
	if (attribName) {
		var numAttribs = attribName.length;
		if (numAttribs > 0) {
			for (var i = 0; i < numAttribs; i++) {
				newElem.setAttribute(attribName[i], attribVal[i]);
			}
		}
	}
	var newDiv = parent.appendChild(newElem);
	return newDiv;
},
/* Method: coordSwap
 * Switch the position of a pair of numbers in a two element array. Used to switch the coords of a point between standard x,y and the method used by the ingame objects where the y and x coords are joined to make a name for a property.
 * 
 * Parameter:
 * coords - array. The array holding the numbers to swap.
 * 
 * Returns:
 * array - The array with the elements swapped.
 */
coordSwap: function (coords) {
	try {
		var t = coords.split('|');
		t.reverse();
		return t.join('|');
	}
	catch (e) {
		return null;
	}
},
/* Method: hypot
 * Calculate the hypotunuse of a right angle triangle. Requires the coords for both vertexes to be in the same order (x,y or y,x).
 * 
 * Parameters:
 * sourceC - coords of one vertex.
 * destC - coords of second vertex.
 * 
 * Returns:
 * Number - The hypotenuse
 */
hypot: function(sourceC, destC) {
	var dx = sourceC[0] - destC[0];
	var dy = sourceC[1] - destC[1];
	dist = Math.sqrt((dx * dx) + (dy * dy));
	return dist;
},

	scriptLoader: function () {
		var scriptList = [dt.server + '/js-sub/idb3.js', dt.server + '/js-sub/inventory.js', dt.server + '/js-sub/military.js', dt.server + '/js-sub/diplo.js', dt.server + '/js-sub/maptools.js', dt.server + '/js-sub/trade.js', dt.server + '/js-sub/data.js', dt.server + '/js-sub/quests.js', dt.server + '/js-sub/crafting.js', dt.server + '/js-sub/mail.js', dt.server + '/js-sub/buildings.js', dt.server + '/js-sub/sovereignty.js', dt.server + '/js-sub/mailsearch.js', dt.server + '/js-sub/production2.js', dt.server + '/js-sub/mutations2.js'];
		for (var i = 0; i < scriptList.length; i++) {
			$.ajax({
				url: scriptList[i],
				dataType: "script",
				cache: false
			}).done(function () {
//		jQuery.cookie("cookie_name", "value", { expires: 7 });
			})
		}
	},
	/* Function: durcbutToggle
 * Toggle the visibility of the entire tools. 
 * 
 */
durcbutToggle: function () {
	var dbut = document.getElementById('durcbut');
	var tlvl = document.getElementById('durcTools3');
	// div.classList.toggle("visible");
	dbut.classList.toggle("see");
	dbut.classList.toggle("nosee");
	tlvl.classList.toggle("see");
	tlvl.classList.toggle("nosee");
},

};
/* *************************************************************************
 * Code
 ************************************************************************* */
// Load the external scripts
dt.scriptLoader();
// Load the css

var winWWidth = window.innerWidth;
var winHeight = window.innerHeight;
var logopos = document.getElementsByClassName('logo').item(0);
var style = window.getComputedStyle(logopos, null);
var left = (parseInt(style.getPropertyValue("left"))) + (parseInt(style.getPropertyValue("width")));


//	var css = document.createElement('link');
//	css.setAttribute('type', 'text/css');
//	css.setAttribute('href', dt.server + '/css/durcTools.css');
//	css.setAttribute('rel', 'stylesheet');
//	document.getElementsByTagName('head')[0].appendChild(css);

var struct = [
	{assignTo:'',parent: 'head', elType: 'link', attrNames: ['type', 'href', 'rel'], attrVals: ['text/css', dt.server + '/css/durcTools3.css', 'stylesheet']},
	{assignTo:'topLevel',parent: 'body', elType: 'div', attrNames: ['id', 'style', 'class'], attrVals: ['durcTools3', 'left:' + left + 'px;top:0px;height:' + winHeight + 'px; border: solid; border-width: 3px; border-color: brown; overflow: auto;', 'nosee']},
	{assignTo:'',parent: '#durcTools3', elType: 'span', attrNames: ['class', 'style', 'onclick', 'title', 'z-index'], attrVals: ['ui-icon ui-icon-closethick', 'right: 0px; top: 0px; position: absolute', 'dt.durcbutToggle()', 'Click to minimize DurcTools v2', '1001']},
	{assignTo:'durcbut',parent: 'body', elType: 'div', attrNames: ['id', 'style', 'class'], attrVals: ['durcbut', 'left:' + left + 'px;top:0px', 'see']},
	{assignTo:'',parent: '#durcbut', elType: 'input', attrNames: ['class', 'onclick', 'title'], attrVals: ['durcbut', 'dt.durcbutToggle()', 'Click to open DurcTools v2']},
	{assignTo:'logo',parent: '#durcTools3', elType: 'div', attrNames: ['id'], attrVals: ['dt_logo']},
	{assignTo:'logArea',parent: '#durcTools3', elType: 'div', attrNames: ['id'], attrVals: ['dt_log']},
{assignTo:'searchDiv',parent:'#durcTools3',elType:'div',attrNames:['id'],attrVals:['dt_rillsearch']},
{assignTo:'',parent:'#dt_rillsearch',elType:'span',attrNames:['style'],attrVals:['text-align:center']},
{assignTo:'',parent:'#dt_rillsearch span',elType:'h4',attrNames:['textContent'],attrVals:['RillSearch']},
{assignTo:'rillSearch',parent:'#dt_rillsearch',elType:'script',attrNames:['src'],attrVals:[dt.server + '/js/rillSearch.js']},
{assignTo:'',parent:'#dt_rillsearch',elType:'gcse:search',attrNames:[],attrVals:[]},
{assignTo:'topMenu',parent:'#durcTools3',elType:'nav',attrNames:['class','id'],attrVals:['see','dt_menu']},
{assignTo:'inputArea',parent:'#durcTools3',elType:'div',attrNames:['class','id'],attrVals:['nosee','dt_input']},
{assignTo:'outputArea',parent:'#durcTools3',elType:'div',attrNames:['class','id'],attrVals:['nosee','dt_output']},
{assignTo:'',parent:'#dt_logo',elType:'img',attrNames:['src','alt','id'],attrVals:[dt.server + '/images/logo2.png', 'Durc Tools', 'dt_logo_img']},
{assignTo:'',parent:'#durcTools3',elType:'div',attrNames:['id','class'],attrVals:['dt_processing','nosee']},
{assignTo:'',parent:'body',elType:'iframe',attrNames:['name', 'id', 'src', 'width', 'height', 'style', 'class'],attrVals:['durctools3help', 'dt_help', dt.server + '/help/DurcToolsHelp.html', '50%', '75%', 'top: 10px; left: 10px; height:' + (winHeight - 10) + 'px;', 'nosee']},
{assignTo:'',parent:'body',elType:'div',attrNames:['name', 'id', 'width', 'height', 'style', 'class'],attrVals:['dt_noteDisplay', 'dt_noteDisplay', '700px', '350px', 'top: 10px; left: 10px;', 'nosee']},
];
//{assignTo:'',parent:'#dt_menu',elType:'ul',attrNames:['id'],attrVals:['firstMenu']},
//{assignTo:'',parent:'#firstMenu',elType:'li',attrNames:['innerHTML'],attrVals:['']},
//{assignTo:'',parent:'',elType:'',attrNames:[],attrVals:[]}

// 	{assignTo:'',parent:'',elType:'',attrNames:[],attrVals:[]},
//	dt.addElement(document.getElementsByTagName('body').item(0), 'iframe', ['name', 'id', 'src', 'width', 'height', 'style', 'class'], ['durctools2bc', 'dt_bci', dt.server + '/html/bc.html', '50%', '75%', 'top: 10px; left: 10px; height:' + (winHeight - 10) + 'px;', 'nosee']);
//	dt.addElement(dt.topLevel, 'div', ['id'], ['dialog']);


for (var i = 0; i < struct.length; i++) {
	dt.addElement(struct[i].assignTo,struct[i].parent, struct[i].elType, struct[i].attrNames, struct[i].attrVals);
};
$( "#dt_menu" ).load( dt.server+"/html/durctools3.html #allmenus" );