/*File: core2.js
 * Defines the objects used in the tools and all the functions that are called 
 * by the tools. By making all the functions etc methods of this it removes any 
 * possibility of conflict with fenctions defined by the in-game scripts.
 * 
 * The files below are loaded to give specific functionality.
 * 
 * idb3.js			- Code to manage the IndexedDB used by the tools.
 * inventory.js		- Code for managing the town and hub inventories.
 * military.js		- Code to collect, store and display information about troops
 *						and commanders.
 * maptools.js		- Scan and collect information from the world map.
 * mail.js			- Allow an in-game mail to be sent to more than one recipient.
 * crafting.js		- Information on crafted items.
 * quests.js			- Tools to help with in-game quests.
 * data.js			- Holds various sets of data that are used in the tools.
 * diplo.js			- This is where the functions for showing the number of diplo 
 *						units will live.
 * trade.js			- This is where the functions for showing the number of trade 
 *						units will live.
 * data.js			- Data objects used by a variety of the functions.
 * quests.js			- Functions relating to the in-game quests.
 * crafting.js		- Functions relating to crafting items.
 * mail.js			- Functions for allowing a message to be sent to more than one
 *						recipient.
 * buildings.js		- Functions for scanning the town map, displaying a towns 
 *						buildings and searching all towns for a building.
 * sovereignty.js	- Holds information on sovereignty bonuses in-game and methods 
 *						to find and store the ones available to each town.
 * mailsearch.js	- Functions for searching the users inbox. Data is stored in a
 *						IndexedDB.
 
 */
/* Class: dt
 * Container for all data and functions relating to the tools.
 * 
 * Properties:
 * server - string. Url for the server. This is the server that all scripts and 
 * resources will be served from.
 * factionScanState - boolean. Indicates if faction troop sightings should be 
 * reported or not.
 * processingHash - boolean. Indicates if the script is in the middle of processing a hash change.
 * db - object. IndexedDB object.
 */
dt = {
	server: 'https://www.durctools3dev.dewt.me.uk',
	factionScanState: false,
	processingHash: false,
	db: null,
	mdb: null,
	oldH: '',
	/*Object: player
	 * Holds all data relating to a player
	 * 
	 * Properties:
	 * name - string. The name of the player.
	 * alliance - integer. The id of the alliance this player belongs to. Is 0 if not in an alliance.
	 * towns - object. Data relating to the towns owned by this player. See <town> for details of the structure.
	 * hubs - object. Data relating to the hubs the player has goods stored at. See <hubs> for details of the structure.
	 * settings - object. Settings for various tools. See <settings> for details of the structure.
	 * toolSettings - object. Position and size settings for each of the tools.
	 */
	player: function (name, alliance, towns, hubs, settings, pres, toolSettings) {
		this.name = name;
		this.alliance = alliance;
		this.towns = towns;
		this.hubs = hubs;
		this.settings = settings;
		this.toolSettings = toolSettings;
	},
	/* Object: town
	 * Holds all data relating to a particular town.
	 * 
	 * Properties:
	 * name - string. The name of the town.
	 * x - integer. X coordinate of the town.
	 * y - integer. Y coordinate of the town.
	 * inv - object. Details of the inventory held by this town. See <inv> for details of the structure.
	 * alembine - object. Status of this towns visits to each of the Allembine sites.
	 * mil - object. Details of the troops currently in this town or being built there and the attack/defence scores for the town. See <mil> for details of the structure.
	 * diplo - object. Details of the diplomatic units currently in this town as well as the diplomatic defence scores for this town. See <diplo> for details of the structure.
	 * trade - details of the trade units currently in this town. See <trade> for details of the structure.
	 */
	town: function (name, x, y, inv, allembine, mil, diplo, trade, prod) {
		this.name = name;
		this.x = x;
		this.y = y;
		this.inv = inv;
		this.allembine = allembine;
		this.mil = mil;
		this.diplo = diplo;
		this.trade = trade;
		this.prod = prod;
	},
	/* Object: hubs
	 * Holds all data relating to a particular trade hub the user has goods and/or trade units at.
	 * 
	 * Properties:
	 * name - The name of the hub.
	 * x - integer. The x coordinate of the hub.
	 * y - integer. The y coordinate of the hub.
	 * traders - unknown. Details of the traders at this hub.
	 * vans - unknown. Details of the caravans at this hub.
	 * inv - object. Details of the items currently held at this hub.
	 */
	hubs: function (name, x, y, traders, vans, inv) {
		this.name = name;
		this.x = x;
		this.y = y;
		this.inv = inv;
		this.traders = traders;
		this.vans = vans;
	},
	/* Object: inv
	 * Holds all data on the items held in the town. The items are organized into the same categories as the tabs at the bottom of the Inventory page.
	 * 
	 * Properties:
	 * maxStore - integer. The maximum storage capacity in this town.
	 * res - object. Details of base and advanced resources held in this town. For each resource a property is created which is an instance of the <item> object.
	 * mounts - object. Details of any specialist mounts held in this town. For each mount type a property is created which is an instance of the <item> object.
	 * weapons - object. Details of any specialist weapons held in this town. For each weapon a property is created which is an instance of the <item> object.
	 * armour - object. Details of any specialist armour held in this town. For each armour a property is created which is an instance of the <item> object.
	 * anatomies - object. Details of any animal parts and hides held in this town. For each anatomy a property is created which is an instance of the <item> object.
	 * minerals - object. Details of any minerals (rare and common) held in this town. For each mineral a property is created which is an instance of the <item> object.
	 * herbs - object. Details of any herbs (rare and common) held in this town. For each herb a property is created which is an instance of the <item> object.
	 * exotic - object. Details of any exotic items held in this town. For each exotic item a property is created which is an instance of the <item> object.
	 */
	inv: function (maxStore, res, mounts, weapons, armour, anatomies, minerals, herbs, exotic) {
		this.maxStore = maxStore;
		this.res = res;
		this.mounts = mounts;
		this.weapons = weapons;
		this.armour = armour;
		this.anatomies = anatomies;
		this.minerals = minerals;
		this.herbs = herbs;
		this.exotic = exotic;
	},
	/* Object: item
	 * Holds all the data for a particular item.
	 * 
	 * Parameters:
	 * name - string. The name of the item.
	 * held - integer. The quantity of this item held.
	 * classes - string. The classes used to display the icon/image for this item.
	 * id - string. String that is used to identify this item in various parts of the DOM.
	 * rate - float. The amount the quantity of this item changes each hour. Is 0 for items that do not have an hourly rate.
	 * 
	 */
	item: function (name, held, classes, id, rate) {
		this.name = name;
		this.held = held;
		this.classes = classes;
		this.id = id;
		this.rate = rate;
	},
	/* Object: trade
	 * Holds the number of each trade unit in a town.
	 * 
	 * Properties:
	 * van - integer. The number of caravans currently in this town.
	 * cotter - integer. The number of cotters currently in this town.
	 * herbalist - integer. The number of herbalists currently in this town.
	 * miner - integer. The number of miners currently in this town.
	 * skinner - integer. The number of skinners currently in this town.
	 * 
	 */
	trade: function (van, cotter, herbalist, miner, skinner, trader) {
		this.van = van;
		this.cotter = cotter;
		this.herbalist = herbalist;
		this.miner = miner;
		this.skinner = skinner;
		this.trader = trader;
	},
	/* Object: diplo
	 * Holds the number of each diplo unit in a town;
	 * 
	 * Properties:
	 * scout - array. The number of T1 and T2 scouts that are in this town.
	 * spy - array. The number of T1 and T2 spies that are in this town.
	 * thief - array. The number of T1 and T2 thieves that are in this town.
	 * saboteur - array. The number of T1 and T2 saboteurs that are in this town.
	 * assassin - array. The number of T1 and T2 assassins that are in this town.
	 * messenger - array. The number of messengers that are in this town.
	 */
	diplo: function (scout, spy, thief, saboteur, assassin, messenger) {
		this.scout = scout;
		this.spy = spy;
		this.thief = thief;
		this.saboteur = saboteur;
		this.assassin = assassin;
		this.messenger = messenger;
	},
	/* Object: mil
	 * Holds all data for mil units and commanders in a town.
	 * 
	 * Properties:
	 * commanders - object. Holds details of the commanders based in this town.
	 * units - object. Details of the units currently available or beinf built in this town.
	 * armies - object. Details of any armies currently in this town.
	 * summary - ?
	 */
	mil: function (commanders, units, armies, summary) {
		this.commanders = commanders;
		this.units = units;
		this.armies = armies;
		this.summary = summary;
	},
	/* Object: commander
	 * Holds all data for a mil commander.
	 * 
	 * Properties:
	 * name - string. The name of this commander.
	 * lvl - integer. The level of this commander.
	 * xp - integer. The current experience of this commander.
	 * self - object. Details of the self bonus levels for this commander.
	 * div - object. Details of the division bonus levels for this commander.
	 * uType - string. The type of unit for this commander (Knight, Charioteer etc)
	 */
	commander: function (name, lvl, xp, self, div, uType) {
		this.name = name;
		this.lvl = lvl;
		this.self = self;// object
		this.div = div;// object
		this.uType = uType;
	},
// Use JSON.parse(JSON.stringify(obj)) to create separate copies of these in each Commander;
	cmdSelf: {atk: 0, spDef: 0, rgDef: 0, infDef: 0, cavDef: 0, cbtSurv: 0, health: 0, heal: 1, escAssas: 0},
	cmdDiv: {spAtk: 0, rgAtk: 0, infAtk: 0, cavAtk: 0, spDef: 0, rgDef: 0, infDef: 0, cavDef: 0, move: 0, carry: 0, magic: 0},
	/* Object: army
	 * Holds all data for an army.
	 * 
	 * Properties:
	 * name - the name of this army.
	 * divisions - object. Details of each of the divisions in this army.
	 */
	army: function (name, divisions) {
		this.name = name;
		this.divisions = divisions;
	},
	/* Object: division
	 * Holds all data for a division.
	 * 
	 * Properties:
	 * name - string. The name of this division.
	 * commander - integer. The id of the commander assigned to this division. Is 0 if no commander.
	 * spear - object. Details of any T1 and T2 spear units in this division.
	 * inf - object. Details of any T1 and T2 infantry units in this division.
	 * ranged - object. Details of any T1 and T2 ranged units in this division.
	 * cav - object. Details of any T1 and T2 cavalry units in this division.
	 * siege - object. Details of any T1 and T2 siege units in this division.
	 */
	division: function (name, commander, spear, inf, ranged, cav, siege) {
		this.name = name;
		this.commander = commander;
		this.spear = spear;
		this.inf = inf;
		this.ranged = ranged;
		this.cav = cav;
		this.siege = siege;
	},
	windowSettings: [],
	/* *************************************************************************
	 * Methods
	 ************************************************************************* */

	/* Method: addElement
	 * Create and append an element with multiple attributes
	 *
	 * Parameters:
	 * assignTo - the name of the property this element should be assigned to.
	 * parent - the element the element is to be appended to. Can be an element reference, an element id or a css selector.
	 * elementType - the type of element to add.
	 * attribVal - array holding the values to be given to the attributes
	 * attribName - array holding the names of the attributes to be added.
	 *
	 * Returns:
	 * The new element.
	 */
	addElement: function (assignTo, parent, elementType, attribNames, attribVals) {
		var newElem = document.createElement(elementType);
		if (attribNames) {
			var numAttribs = attribNames.length;
			if (numAttribs > 0) {
				for (var i = 0; i < numAttribs; i++) {
					switch (attribNames[i]) {
						case 'textContent':
							newElem.textContent = attribVals[i];
							break;
						case 'innerHTML':
							newElem.innerHTML = attribVals[i];
							break;
						default:
							newElem.setAttribute(attribNames[i], attribVals[i]);
							break;
					}
				}
			}
		}
		try {
			if (document.querySelector(parent) !== null) {
				var newDiv = document.querySelector(parent).appendChild(newElem);
			} else {
				var newDiv = document.getElementById(parent).appendChild(newElem);
			}
		}
		catch (e) {
			var newDiv = parent.appendChild(newElem);
		}
		if (assignTo !== '') {
			dt[assignTo] = newDiv;
		}
		return newDiv;
	},
	/* Method: addElement2
	 * Create and append an element with multiple attributes
	 *
	 * Parameters:
	 * parent - the element the element is to be appended to
	 * elementType - the type of element to add.
	 * attribVal - array holding the values to be given to the attributes
	 * attribName - array holding the names of the attributes to be added.
	 *
	 * Returns:
	 * The new element.
	 */
	addElement2: function (parent, elementType, attribName, attribVal) {
		var newElem = document.createElement(elementType);
		if (attribName) {
			var numAttribs = attribName.length;
			if (numAttribs > 0) {
				for (var i = 0; i < numAttribs; i++) {
					newElem.setAttribute(attribName[i], attribVal[i]);
				}
			}
		}
		var newDiv = parent.appendChild(newElem);
		return newDiv;
	},
	/* Function: logEntry
	 * Add an entry to the log div
	 * 
	 * Parameter:
	 * msg - string. The message to be added.
	 */
	logEntry: function (msg) {
		// Get server time
		var serverTimeArray = document.querySelector("#serverDate").textContent.split(" ");
		serverTime = serverTimeArray[3];
		if(serverTimeArray[3] == ''){
			serverTime = serverTimeArray[4];
		}
		console.log("serverDate: "+document.querySelector("#serverDate").textContent);
		if(serverTime == ''){
			console.log("debug: logEntry serverTime is null...")
		}
		var parent = document.querySelector('#dt_log');
		newElement = document.createElement("p");
		newElement.textContent = serverTime+" - ("+dt[PlayerId].towns[CurrentTown].name+"):"+msg;
		parent.appendChild(newElement);
//		newElement.scrollIntoView(true);
		var topPos = newElement.offsetTop;
		parent.scrollTop = topPos;
	},
	/* Method: numberFormat
	 * Format a number to display it in the format  d, ddd.ddK or ddd.ddM
	 * 
	 * Parameter:
	 * orig - float. The number to be formatted.
	 * 
	 * Returns: string or integer.
	 */
	numberFormat: function (orig) {
		switch (true) {
			case (orig > 999999):
				var formatted = (orig / 1000000).toFixed(2);
				formatted = formatted.toString() + 'M';
				break;
			case (orig > 999):
				var formatted = (orig / 1000).toFixed(2);
				formatted = formatted.toString() + 'k';
				break;
			default:
				formatted = parseInt(orig);
		}
		return formatted;
	},
	secondsToString: function (seconds) {
		var numdays = Math.floor(seconds / 86400);
		var numhours = Math.floor((seconds % 86400) / 3600);
		var numminutes = Math.floor(((seconds % 86400) % 3600) / 60);
		var numseconds = ((seconds % 86400) % 3600) % 60;
		return numdays + " days " + numhours + " hours " + numminutes + " minutes " + numseconds + " seconds";
	},
	secondsToString2: function (seconds) {
		seconds = Math.abs(seconds);
		retval = '';
		var numdays = Math.floor(seconds / 86400);
		var numhours = (seconds % 86400/3600).toFixed(1);
		var numminutes = Math.floor(((seconds % 86400) % 3600) / 60);
		var numseconds = ((seconds % 86400) % 3600) % 60;
		if (numdays > 0) {
			retval = numdays + "d ";
		}
		if (numhours > 0) {
			retval = retval + numhours + "h";
		}
		if (retval == '') {
			retval = '-';
		}
//return numdays + "d " + numhours + "h";
		return retval.trim();
	},
	hoursToString: function (hours) {
		var numdays = Math.floor(hours / 24);
		var numhours = Math.floor((hours % 24) % 24);
		return numdays + " days " + numhours + " hours ";
	},
	/* Function: numberFormat2
	 * Format a number to display it in the format  d, dddK or dddM
	 * 
	 * Parameter:
	 * orig - float. The number to be formatted.
	 * 
	 * Returns: string or integer.
	 */
	numberFormat2: function (orig) {
		switch (true) {
			case (orig > 999999):
				var formatted = (orig / 1000000).toFixed(0);
				formatted = formatted.toString() + 'M';
				break;
			case (orig > 999):
				var formatted = (orig / 1000).toFixed(0);
				formatted = formatted.toString() + 'k';
				break;
			default:
				formatted = parseInt(orig);
		}
		return formatted;
	},
	/* Function: arrayMin
	 * Return the smallest value in the supplied array.
	 * 
	 * Parameter:
	 * numArray - array. The array to process.
	 * 
	 * Return:
	 * Number.
	 */
	arrayMin: function (numArray) {
		return Math.min.apply(null, numArray);
	},
	/* Function: arrayMax
	 * Return the largest value in the supplied array.
	 * 
	 * Parameter:
	 * numArray - array. The array to process.
	 * 
	 * Return:
	 * Number.
	 */
	arrayMax: function (numArray) {
		return Math.max.apply(null, numArray);
	},
	/* Method: sortObject
	 * Sort an object.
	 * 
	 * Parameter:
	 * obj - object. The object to be sorted.
	 * 
	 * Returns:
	 * The sorted object.
	 */
	sortObject: function (obj) {
		if (obj !== 'undefined') {
			return Object.keys(obj).sort().reduce(function (result, key) {
				result[key] = obj[key];
				return result;
			}, {});
		}
	},
	/* Method: coordSwap
	 * Switch the position of a pair of numbers in a two element array. Used to switch the coords of a point between standard x,y and the method used by the ingame objects where the y and x coords are joined to make a name for a property.
	 * 
	 * Parameter:
	 * coords - array. The array holding the numbers to swap.
	 * 
	 * Returns:
	 * array - The array with the elements swapped.
	 */
	coordSwap: function (coords) {
		try {
			var t = coords.split('|');
			t.reverse();
			return t.join('|');
		}
		catch (e) {
			return null;
		}
	},
	/* Method: hypot
	 * Calculate the hypotunuse of a right angle triangle. Requires the coords for both vertexes to be in the same order (x,y or y,x).
	 * 
	 * Parameters:
	 * sourceC - coords of one vertex.
	 * destC - coords of second vertex.
	 * 
	 * Returns:
	 * Number - The hypotenuse
	 */
	hypot: function (sourceC, destC) {
		var dx = sourceC[0] - destC[0];
		var dy = sourceC[1] - destC[1];
		dist = Math.sqrt((dx * dx) + (dy * dy));
		return dist;
	},
	/* Function: newHash
	 * Identify the change to the hash and/or town and call the relevant function after a delay of 2 seconds (to allow the in-game scripts time to finish up-dating the DOM).
	 * 
	 * Dependant on the change various functions will be called to update the resource holdings, scan the map etc.
	 * 
	 */
	newHash: function () {
		if (dt.processingHash === false) {
			dt.processingHash = true;
			$(document).ready(function newHashLoaded() {
				var newHash = window.location.hash;
				var newTown = CurrentTown;
				if (dt.currentTown !== CurrentTown) {
					dt.currentTown = CurrentTown;
					dt.getTopRes();
					dt.getAdvRes();
					dt.getBottomInv();
				}
				if (newHash.indexOf('?')) {
					var bits = newHash.split('?');
					newHash = bits[0];
				}
				var bits = newHash.split('/');
				switch (bits[1]) {
					case "World":
						if (bits[2] === 'Map') {
							dt.currentHash = window.location.hash;
							if (dt[PlayerId].settings.factionScanState === true) {
								var timeoutID = window.setTimeout(dt.scanFactionTroops, 2000);
							}
							dt.scanMap();
						}
						break;
					case "Communication":
						if (bits[2] === 'Mail') {
//					dt.mailsearch(dt.scanMail);
						}
						break;
					case "Town":
						if (bits[2] === 'Inventory') {
							dt.currentHash = window.location.hash;
							try {
								var timeoutID = window.setTimeout(dt.getTownInventory, 2000);
							}
							catch (e) {
							}
						}
						;
						switch (bits[2]) {
							case "Inventory":
								dt.currentHash = window.location.hash;
								try {
									var timeoutID = window.setTimeout(dt.getTownInventory, 2000);
								}
								catch (e) {
								}
								break;
							case "Production":
								dt.currentHash = window.location.hash;
								try {
									dt.getProdSchedule();
								}
								catch (e) {
								}
								break;
							case "Map":
								dt.currentHash = window.location.hash;
								dt.buildings.bldgScan();
								break;
						}
						break;
					case "Military":
						switch (bits[2]) {
							case "Movements":
								dt.currentHash = window.location.hash;
								dt.scanMilMov();
								break;
							case "Armies":
								dt.currentHash = window.location.hash;
								dt.scanMilArmy();
								break;
							case "Commanders":
								dt.currentHash = window.location.hash;
								dt.scanMilCommander();
								break;
							case undefined:
								dt.currentHash = window.location.hash;
								var timeoutID = window.setTimeout(dt.military.scanMilOverview, 2000);
								break;
							default:
								// Do nothing
						}
						;
						break;
					case "Diplomatic":
						switch (bits[2]) {
							case "Movements":
								dt.currentHash = window.location.hash;
								dt.scanDiploMov();
								break;
							case undefined:
								dt.currentHash = window.location.hash;
								dt.scanDiploOverview();
								break;
							default:
						}
						;
						break;
					case "Trade":
						switch (bits[2]) {
							case "Movements":
								dt.currentHash = window.location.hash;
								dt.scanTradeMov();
								break;
							case "HubInventory":
								// https://elgea.illyriad.co.uk/#/Trade/HubInventory?HubId=1&Subtype=0
								dt.currentHash = window.location.hash;
								var timeoutID = window.setTimeout(dt.getHubList, 1000);
								var timeoutID = window.setTimeout(dt.getHubInv, 2000);
								dt.invent.getHubInv();
								break;
							case undefined:
								dt.currentHash = window.location.hash;
								dt.scanTradeOverview();
								break;
							default:
						}
						break;
				}
			});
			dt.processingHash = false;
		}
	},
	/* Method: scriptLoader
	 * Load the scripts held in the scriptList array.
	 */
	scriptLoader: function () {
		var scriptList = [dt.server + '/js/tablesort.js',
			dt.server + '/js-sub/idb3.js',
			dt.server + '/js-sub/inventory.js',
			dt.server + '/js-sub/military.js',
			dt.server + '/js-sub/diplo.js',
			dt.server + '/js-sub/maptools.js',
			dt.server + '/js-sub/trade.js',
			dt.server + '/js-sub/data.js',
			dt.server + '/js-sub/quests.js',
			dt.server + '/js-sub/crafting.js',
			dt.server + '/js-sub/mail.js',
			dt.server + '/js-sub/buildings.js',
			dt.server + '/js-sub/sovereignty.js',
			dt.server + '/js-sub/mailsearch.js',
			dt.server + '/js-sub/production.js',
			dt.server + '/js-sub/preferences.js',
			dt.server + '/js-sub/misc.js',
			dt.server + '/js-sub/elementChecker.js',
			dt.server + '/js/intercept2.js'];
//			dt.server + '/js-sub/mutations.js',
		for (var i = 0; i < scriptList.length; i++) {
			$.ajax({
				url: scriptList[i],
				dataType: "script",
				cache: false
			}).done(function () {
//		jQuery.cookie("cookie_name", "value", { expires: 7 });
			})
		}
	},
	/* Function: durcbutToggle
	 * Toggle the visibility of the entire tools. 
	 * 
	 */
	durcbutToggle: function () {
		var dbut = document.getElementById('durcbut');
		var tlvl = document.getElementById('durcTools3');
		// div.classList.toggle("visible");
		dbut.classList.toggle("see");
		dbut.classList.toggle("nosee");
		tlvl.classList.toggle("see");
		tlvl.classList.toggle("nosee");
	},
	/* Method: showTool
	 * Display the dialog for the tool.
	 */
	showTool: function (id) {
		var winHeight = window.innerHeight;
		var maxHt = parseInt(0.9 * winHeight);
		var winWidth = window.innerWidth;
		var maxWd = parseInt(0.75 * winWidth);
		if (dt[PlayerId].toolSettings[id] !== undefined) {
			dt[PlayerId].toolSettings[id].maxHt = maxHt;
			dt[PlayerId].toolSettings[id].maxWd = maxWd;
			var me = dt[PlayerId].toolSettings[id];
		} else {
			me = {width: 'auto', maxWd: maxWd, height: 'auto', maxHt: maxHt, pos: {left: 'center', top: 20}};
		}
		$(function () {
			$(id).dialog({
				"width": me.width,
				"maxWidth": me.maxWd,
				"height": me.height,
				"maxHeight": me.maxHt,
				resizeStop: function (event, ui) {
					me.width = ui.size.width;
					me.height = ui.size.height;
					me.pos = ui.position;
					dt[PlayerId].toolSettings[id] = me;
					var key = PlayerId + ':toolSettings';
					locStorage.setItem(key, JSON.stringify(dt[PlayerId].toolSettings));
				},
				dragStop: function (event, ui) {
					me.pos = ui.position;
					dt[PlayerId].toolSettings[id] = me;
					var key = PlayerId + ':toolSettings';
					locStorage.setItem(key, JSON.stringify(dt[PlayerId].toolSettings));
				}
			});
		});
		// 
		if (dt[PlayerId].toolSettings[id] == undefined) {
			this.adjustTool(id);
		} else {
			this.posTool(id);
		}
	},
	/* Method: adjustTool
	 * Check if the tool width and height are within the maximum values and adjust as needed.
	 * 
	 * Parameter:
	 * id - string. The css selector for the tool div.
	 */
	adjustTool: function (id) {
		var winHeight = window.innerHeight;
		var maxHt = parseInt(0.9 * winHeight);
		var winWidth = window.innerWidth;
		var maxWd = parseInt(0.75 * winWidth);
		var el = document.querySelector('div.ui-dialog > ' + id).parentElement;

		var intElemOffsetHeight = el.offsetHeight;
		var intElemOffsetWidth = el.offsetWidth;
		var currHeight = $(id).dialog("option", "height");
		if (intElemOffsetHeight > maxHt) {
			$(id).dialog("option", "height", maxHt);
		}
		if (intElemOffsetWidth > maxWd) {
			$(id).dialog("option", "width", maxWd);
		}
	},
	/* Method: posTool
	 * Change the position of the tool to the stored coords and size.
	 * 
	 * Parameter:
	 * id - string. The css selector for the tool div.
	 */
	posTool: function (id) {
		var pel = document.querySelector(id).parentElement;
		pel.style.left = dt[PlayerId].toolSettings[id].pos.left + "px";
		pel.style.top = dt[PlayerId].toolSettings[id].pos.top + "px";
		pel.style.width = dt[PlayerId].toolSettings[id].width + "px";
		pel.style.height = dt[PlayerId].toolSettings[id].height + "px";
	},
	/* Method: removeRows
	 * Remove all the rows from the body of a table.
	 * 
	 * Parameter:
	 * tbl - object reference. The table to remove the rows from.
	 */
	removeRows: function (tbl) {
		var bdy = tbl.querySelector('tbody');
		if (bdy.hasChildNodes()) {
			while (bdy.firstChild) {
				bdy.removeChild(bdy.firstChild);
			}
		}
	},
	/* Function: replaceAll
	 * Replaces all occurences of needle in a string with the specified string. It 
	 * works by first splitting str on each occurence of the sub-string in search 
	 * and then joining the resulting array elements with the string replace 
	 * in-between them. This will work because split removes the contents of the 
	 * sub-string.
	 * 
	 *  Parameters: 
	 * str - string. The string to search in.
	 * search - string. The sub-string to be replaced.
	 * replace - string. The string to replace search with.
	 * 
	 * Returns:
	 * string. The new string.
	 */
	replaceAll: function (str, search, replace) {
		var newStr = str.split(search).join(replace);
		return newStr;
	},
	/* Function: getTownList
	 * Parse the optTown element to get a list of all the towns owned by the current player and either update or create a <town> object to hold the data.
	 * 
	 */
	getTownList: function () {
		var townDivs = document.getElementsByClassName('cityMenu');
		var numTowns = townDivs.length;
		dt[PlayerId].towns.list = [];
		dt[PlayerId].towns.numTowns = numTowns;
		var list = [];
		for (var i = 0; i < numTowns; i++) {
			var townId = townDivs.item(i).getAttribute('data-townid');
			var name = townDivs.item(i).getElementsByTagName('div').item(0).textContent;
			name = name.replace("- Capital", "");
			name = name.trim();
			var coords = townDivs.item(i).getElementsByTagName('div').item(1).textContent;
			var coordArray = coords.split(' ');
			if (dt[PlayerId].towns.hasOwnProperty(townId) === false) {
				dt[PlayerId].towns[townId] = new dt.town(name, coordArray[0].trim(), coordArray[2].trim(), new dt.inv(0, {}, {}, {}, {}, {}, {}, {}, {}), '', '', '', '');
			} else {
				dt[PlayerId].towns[townId].name = name;
				dt[PlayerId].towns[townId].x = coordArray[0].trim();
				dt[PlayerId].towns[townId].y = coordArray[1].trim();
			}
			dt[PlayerId].towns.list.push(townId);
		}
		dt[PlayerId].towns.list.sort();
	},
	/* Function: toggleFactionScanState
	 * Toggle scanning and reporting of Faction troops on the world map.
	 */
	toggleFactionScanState: function () {
		var factionchk = document.querySelector("#factionscan");
		if (dt[PlayerId].settings.factionScanState === true) {
			dt[PlayerId].settings.factionScanState = false;
		} else {
			dt[PlayerId].settings.factionScanState = true;
		}
		var key = PlayerId + ':settings';
		locStorage.setItem(key, JSON.stringify(dt[PlayerId].settings));
	},
	townChanged: function () {
		if (dt.currentTown !== CurrentTown) {
			dt.hashChanged();
		}
	},
	/* Method: getURLParams
	 * Build an array of any parameters passed in the url. Empty array if none.
	 * Each parameter is returned as a two element array (key, value).
	 * 
	 * Returns:
	 *  {Array|params}
	 */
	getURLParams: function () {
		params = [];
		var h = location.hash;
		var paramString = h.split("?")[1];
		if (!paramString) {
			return params;
		}
		var pArray = paramString.split("&");
		for (var i in pArray) {
			params.push(pArray[i].split("="));
		}
		return params;
	},
	/* Method: hashChanged
	 * Called whenever the page hash changes. Builds an array of the parts of the hash including any parameters.
	 * Depending on the values in the array calls the relevant function.
	 * 
	 * No matter what the hash is it will call the method to get the base and 
	 * advanced resources for the current town.
	 */
	hashChanged: function () {
		var rubbish = "";
//		$(document).ready(function () {
		var hashParts = location.hash.split("?")[0].split("/");
		var params = dt.getURLParams();
		hashParts.push(params);
		var newTown = CurrentTown;
		if (dt.currentTown !== CurrentTown) {
			dt.currentTown = CurrentTown;
		}
		var rubbish = '';
		switch (hashParts[1]) {
			case "World":
				// Possible values for hashParts[2]: Map, Quests, Tournaments, Herald(several sub sections)
				if (hashParts[2] === "Map") {
					if (dt[PlayerId].settings.factionScanState === true) {
						dt.scanFactionTroops();
					}
				}

				break;
			case "Town":
				// Possible values for hashParts[2]: City, Map, Castle(several sub sections), Resources, Production, Sovereignty, Inventory, Growth, UnitProduction(hashParts[3] defines which type (mil, trade, diplo) are being produced))
				switch (hashParts[2]) {
					case "City":
						break;
					case "Map":
						// Get building levels
						dt.buildings.bldgScan();
						break;
					case "Production":
						// Get production queues
						dt.production.getProdSchedule();
						break;
					case "Sovereignty":
						// Get details of sovereignty claims and highlight any that can have the building upgraded.
						dt.sov.getSovDetails();
						break;
					case "Inventory":
						// Get the maximum storage capacity and the currently selected sub-inventory
						dt.invent.getTownInventory();
						break;
					case "BuildingProduction":
						// hashParts[3] holds the id of the building being viewed
						dt.production.getBldgProd(hashParts[3]);
						break;
				}
				break;
			case "Military":
				// Possible values for hashParts[2]: none, Movements, Sieges, Orders, Armies, Commanders
				switch (hashParts[2]) {
					case "Commanders":
						dt.military.scanMilCommander();
						break;
					case "Movements":
						dt.military.scanMilMov();
						break;
					case "Sieges":
						dt.military.scanSieges();
						break;
					case "Orders":
						dt.military.addOccupyMax();
						break;
					case "Armies":
						dt.military.scanMilArmy();
						break;
					default:
						dt.military.scanMilOverview();
						break;
				}
				break;
			case "Diplomatic":
				// Possible values for hashParts[2]: none, Movements, Orders
				// Currently nothing being called from here
				break;
			case "Trade":
				// Possible values for hashParts[2]: none, Movements, Orders, Markets, HubInventory, HubBuyOffers, HubSellOffers, HubInProgress, HubOrders, HubMovements, HubTransactions
				switch (hashParts[2]) {
					case "Movements":
						break;
					case "Orders":
						break;
					case "Markets":
						dt.trade.tradeMarkets();
						break;
					case "HubInventory":
						dt.invent.getHubInv(params);
						break;
					default:
						// Catch everything else
						break;
				}
				break;
			case "Research":

				break;
			case "Magic":

				break;
			case "Player":

				break;
			case "Alliance":

				break;
			case "Prestige":

				break;
			case "Communication":
				// Possible values for hashParts[2]: Mail, Notifications, Chat
				switch (hashParts[2]) {
					case "Mail":
						dt.mailsearch(dt.scanMail);
						break;
				}
				break;
		}
		// Call the methods to collect any inventory info on the page.
		console.log("Trying inventory gathering...");
		try {
			dt.invent.getTopRes();
		}
		catch (e) {
			console.log("error in call to inventory gathering at the end of hashChanged...");
			console.log("debug: " + e.message);
			var rubbish = '';
		}
		try {
			dt.invent.getAdvRes();
		}
		catch (e) {
			console.log("error in call to inventory gathering at the end of hashChanged...");
			console.log("debug: " + e.message);
			var rubbish = '';
		}
		try {
			dt.invent.getBottomInv();
		}
		catch (e) {
			console.log("error in call to inventory gathering at the end of hashChanged...");
			console.log("debug: " + e.message);
			var rubbish = '';
		}

//			dt.invent.getTownInventory();

	},
};
/* *************************************************************************
 * Code
 ************************************************************************* */

// Load the external scripts
dt.scriptLoader();

var winWWidth = window.innerWidth;
var winHeight = window.innerHeight;
var logopos = document.getElementsByClassName('logo').item(0);
var style = window.getComputedStyle(logopos, null);
var left = (parseInt(style.getPropertyValue("left"))) + (parseInt(style.getPropertyValue("width")));

var struct = [
	{assignTo: '', parent: 'head', elType: 'link', attrNames: ['type', 'href', 'rel'], attrVals: ['text/css', dt.server + '/css/durcTools3.css', 'stylesheet']},
	{assignTo: 'topLevel', parent: 'body', elType: 'div', attrNames: ['id', 'style', 'class'], attrVals: ['durcTools3', 'left:' + left + 'px;top:0px;height:' + winHeight + 'px; border: solid; border-width: 3px; border-color: brown; overflow: auto;', 'nosee']},
	{assignTo: 'prodAlarm', parent: 'body', elType: 'img', attrNames: ['alt', 'id', 'class', 'src'], attrVals: ['Prod queue ended', 'dt_prodAlarm', 'nosee', 'https://www.durctools2dev.dewt.me.uk/images/prodAlert.gif']}
];

for (var i = 0; i < struct.length; i++) {
	dt.addElement(struct[i].assignTo, struct[i].parent, struct[i].elType, struct[i].attrNames, struct[i].attrVals);
}
;
// Load the html for the tools
$("#durcTools3").load(dt.server + "/html/durctools3.html");
var durcbut = dt.addElement('', 'body', 'div', ['id', 'style', 'class'], ['durcbut', 'left:' + left + 'px;top:0px', 'see']);
durcbut.innerHTML = '<input class="durcbut" onclick="dt.durcbutToggle()" title="Click to open DurcTools v2">';
dt[PlayerId] = new dt.player(CurPlayerName, YourAllianceId, {}, {}, {factionScanState: false, alertAudio: false, alertVisual: false, alertFreq: 10}, 0, []);
// Get the id and name of each of the player towns
dt.getTownList();
var tList = dt[PlayerId].towns.list;
//	dt.init();

dt.currentTown = CurrentTown;
// Load the settings for this player
dt[PlayerId].settings = JSON.parse(locStorage.getItem([PlayerId] + ':settings'));
if (dt[PlayerId].settings === null) {
	dt[PlayerId].settings = {};
}
// Load the toolSettings for this player
dt[PlayerId].toolSettings = JSON.parse(locStorage.getItem([PlayerId] + ':toolSettings'));
if (dt[PlayerId].toolSettings === null) {
	dt[PlayerId].toolSettings = {};
}
// Loop through each of this players towns
var numTowns = dt[PlayerId].towns.list.length;
var tList = dt[PlayerId].towns.list;
for (var i = 0; i < numTowns; i++) {
	// Load any inventories in local storage
	dt[PlayerId].towns[tList[i]].inv = JSON.parse(locStorage.getItem(tList[i] + ':inv'));
	if (dt[PlayerId].towns[tList[i]].inv === null) {
		dt[PlayerId].towns[tList[i]].inv = new dt.inv(0, {}, {}, {}, {}, {}, {}, {}, {});
	}
	// Load any military info in local storage
	dt[PlayerId].towns[tList[i]].mil = JSON.parse(locStorage.getItem(tList[i] + ':mil'));
	if (dt[PlayerId].towns[tList[i]].mil === null) {
		dt[PlayerId].towns[tList[i]].mil = new dt.mil({}, {spear: {t1: {pool: 0, build: 0, armies: 0}, t2: {pool: 0, build: 0, armies: 0}}, inf: {t1: {pool: 0, build: 0, armies: 0}, t2: {pool: 0, build: 0, armies: 0}}, range: {t1: {pool: 0, build: 0, armies: 0}, t2: {pool: 0, build: 0, armies: 0}}, cav: {t1: {pool: 0, build: 0, armies: 0}, t2: {pool: 0, build: 0, armies: 0}}, siege: {t1: {pool: 0, build: 0, armies: 0}, t2: {pool: 0, build: 0, armies: 0}}}, {}, {atk: 0, spearDef: 0, infDef: 0, bowDef: 0, cavDef: 0});
	}
	// Load any diplo info in local storage
	dt[PlayerId].towns[tList[i]].diplo = JSON.parse(locStorage.getItem(tList[i] + ':diplo'));
	if (dt[PlayerId].towns[tList[i]].diplo === null) {
		dt[PlayerId].towns[tList[i]].diplo = new dt.diplo({t1: [], t2: []}, {t1: [], t2: []}, {t1: [], t2: []}, {t1: [], t2: []}, {t1: [], t2: []}, []);
	}
	// Load any trade info in local storage
	dt[PlayerId].towns[tList[i]].trade = JSON.parse(locStorage.getItem(tList[i] + ':trade'));
	if (dt[PlayerId].towns[tList[i]].trade === null) {
		dt[PlayerId].towns[tList[i]].trade = new dt.trade([0, 0, 0, 0, 0], [0, 0, 0, 0, 0], [0, 0, 0, 0, 0], [0, 0, 0, 0, 0], [0, 0, 0, 0, 0], [0, 0, 0, 0, 0]);
	}
	// Load any allembine quest info in local storage
	dt[PlayerId].towns[tList[i]].allembine = JSON.parse(locStorage.getItem(tList[i] + ':allem'));
	if (dt[PlayerId].towns[tList[i]].allembine === null) {
		dt[PlayerId].towns[tList[i]].allembine = {1: {a: 'false'}, 2: {a: 'false'}, 3: {a: 'false'}, 4: {a: 'false', b: 'false'}, 5: {a: 'false'}, 6: {a: 'false'}, 7: {a: 'false', b: 'false'}, 8: {a: 'false', b: 'false'}, 9: {a: 'false'}, 10: {a: 'false'}, 11: {a: 'false'}, 12: {a: 'false'}, 13: {a: 'false', b: 'false'}, 14: {a: 'false'}, 15: {a: 'false', b: 'false'}, 16: {a: 'false'}, 17: {a: 'false'}, 18: {a: 'false'}, 19: {a: 'false', b: 'false'}, 20: {a: 'false'}};
	}
	// Load any prod queue info in local storage
	dt[PlayerId].towns[tList[i]].prod = JSON.parse(locStorage.getItem(tList[i] + ':prod'));
	if (dt[PlayerId].towns[tList[i]].prod === null) {
		dt[PlayerId].towns[tList[i]].prod = {};
	}
	// Load any building lvl info in local storage
	dt[PlayerId].towns[tList[i]].plots = JSON.parse(locStorage.getItem(tList[i] + ':plots'));
	if (dt[PlayerId].towns[tList[i]].plots === null) {
		dt[PlayerId].towns[tList[i]].plots = {resource: {1: "", 2: "", 3: "", 4: "", 5: "", 6: "", 7: "", 8: "", 9: "", 10: "", 11: "", 12: "", 13: "", 14: "", 15: "", 16: "", 17: "", 18: "", 19: "", 20: "", 21: "", 22: "", 23: "", 24: "", 25: ""}, internal: {0: "", 1: "", 2: "", 3: "", 4: "", 5: "", 6: "", 7: "", 8: "", 9: "", 10: "", 11: "", 12: "", 13: "", 14: "", 15: "", 16: "", 17: "", 18: "", 19: "", 20: "", 21: "", 22: "", 23: "", 24: "", 25: ""}};
	}
	// Load any sov level info in local storage
	dt[PlayerId].towns[tList[i]].sov = JSON.parse(locStorage.getItem(tList[i] + ':sov'));
	if (dt[PlayerId].towns[tList[i]].sov === null) {
		dt[PlayerId].towns[tList[i]].plots = {};
	}
}
// Load any trade hub info in local storage
dt[PlayerId].hubs = JSON.parse(locStorage.getItem(PlayerId + ':hubs'));
if (dt[PlayerId].hubs === null) {
	dt[PlayerId].hubs = {};
}

// Get the inventory for the current town
//try {
//	dt.getTownInventory();
//}
//catch (e) {
//}
window.onhashchange = dt.hashChanged;
// primaryObserver = dt.mutations.setup();
